pub struct ConnectTokenEncoder { /* private fields */ }
Represents a private key used to create JWT tokens for use with App Store Connect.

See https://developer.apple.com/documentation/appstoreconnectapi/creating_api_keys_for_app_store_connect_api and https://developer.apple.com/documentation/appstoreconnectapi/generating_tokens_for_api_requests for more details.

This entity holds the necessary metadata to issue new JWT tokens.

App Store Connect API tokens/JWTs are derived from:

  • A key identifier. This is a short alphanumeric string like DEADBEEF42.
  • An issuer ID. This is likely a UUID.
  • A private key. Likely ECDSA.

All these are issued by Apple. You can log in to App Store Connect and see/manage your keys at https://appstoreconnect.apple.com/access/api.

Implementations

Construct an instance from an EncodingKey instance.

This is the lowest level API and ultimately what all constructors use.

Construct an instance from a DER encoded ECDSA private key.

Create a token from a PEM encoded ECDSA private key.

Create a token from a PEM encoded ECDSA private key in a filesystem path.

Attempt to construct in instance from an API Key ID.

e.g. DEADBEEF42. This looks for an AuthKey_<id>.p8 file in default search locations like ~/.appstoreconnect/private_keys.

Mint a new JWT token.

Using the private key and key metadata bound to this instance, we issue a new JWT for the requested duration.