pub struct UnifiedApiKey { /* private fields */ }
Represents all metadata for an App Store Connect API Key.

This is a convenience type to aid in the generic representation of all the components of an App Store Connect API Key. The type supports serialization so we save as a single file or payload to enhance usability (so people don’t need to provide all 3 pieces of the API Key for all operations).

Implementations

Construct an instance from constitute parts and a PEM encoded ECDSA private key.

This is what you want to use if importing a private key from the file downloaded from the App Store Connect web interface.

Construct an instance from serialized JSON.

Construct an instance from a JSON file.

Serialize this instance to a JSON object.

Write this instance to a JSON file.

Since the file contains sensitive data, it will have limited read permissions on platforms where this is implemented. Parent directories will be created if missing using default permissions for created directories.

Permissions on the resulting file may not be as restrictive as desired. It is up to callers to additionally harden as desired.