pub enum CodeSigningSlot {
Show 17 variants
CodeDirectory,
Info,
RequirementSet,
ResourceDir,
Application,
Entitlements,
RepSpecific,
EntitlementsDer,
AlternateCodeDirectory0,
AlternateCodeDirectory1,
AlternateCodeDirectory2,
AlternateCodeDirectory3,
AlternateCodeDirectory4,
Signature,
Identification,
Ticket,
Unknown(u32),
}Expand description
A well-known slot within code signing data.
Variants
CodeDirectory
Info
Info.plist.
RequirementSet
Designated requirements.
ResourceDir
Digest of CodeRequirements file (used in bundles).
Application
Application specific slot.
Entitlements
Entitlements XML plist.
RepSpecific
Reserved for disk images.
EntitlementsDer
Entitlements DER encoded plist.
AlternateCodeDirectory0
Alternative code directory slot #0.
Used for expressing a code directory using an alternate digest type.
AlternateCodeDirectory1
AlternateCodeDirectory2
AlternateCodeDirectory3
AlternateCodeDirectory4
Signature
CMS signature.
Identification
Ticket
Notarization ticket.
Unknown(u32)
Implementations
sourceimpl CodeSigningSlot
impl CodeSigningSlot
sourcepub fn has_external_content(&self) -> bool
pub fn has_external_content(&self) -> bool
Whether this slot has external data (as opposed to provided via a blob).
sourcepub fn is_alternative_code_directory(&self) -> bool
pub fn is_alternative_code_directory(&self) -> bool
Whether this slot is for holding an alternative code directory.
sourcepub fn is_code_directory_specials_expressible(&self) -> bool
pub fn is_code_directory_specials_expressible(&self) -> bool
Whether this slot’s digest is expressed in code directories list of special slot digests.