Module remote_signing

Module remote_signing 

Remote signing support.

Modules

session_negotiation
Session establishment and crypto code for remote signing protocol.

Structs

InitiatorClient
A client fulfilling the role of the initiator.
PairedClient
A remote signing client that has joined a session and is ready to exchange messages.
SigningClient
UnjoinedSigningClient
A client for the remote signing protocol that has not yet joined a session.

Enums

RemoteSignError
An error specific to remote signing.

Constants

DEFAULT_SERVER_URL
URL of default server to use.

Type Aliases

SessionInfoCallback
A function that receives session information.