pub trait SessionJoinPeerPreJoin {
    // Required methods
    fn register_state(
        &mut self,
        state: SessionJoinState
    ) -> Result<(), RemoteSignError>;
    fn join_context(
        self: Box<Self>
    ) -> Result<SessionJoinContext, RemoteSignError>;
}
A peer that joins sessions in a state before it has spoken to the server.

Required Methods

source

fn register_state( &mut self, state: SessionJoinState ) -> Result<(), RemoteSignError>

Register additional state with the peer.

This is used as a generic way to import implementation-specific state that enables the peer join to complete.

source

fn join_context(self: Box<Self>) -> Result<SessionJoinContext, RemoteSignError>

Obtain information needed to join to a session.

Consumes self because joining should be a one-time operation.

Implementors