SessionJoinString

Trait SessionJoinString 

pub trait SessionJoinString<'de>: CborDecode<'de, ()> + CborEncode<()> {
    // Required method
    fn scheme() -> &'static str;

    // Provided method
    fn to_bytes(&self) -> Result<Vec<u8>, RemoteSignError> { ... }
}
Common behaviors for a session join string.

Implementations must also implement Encode, which will emit the CBOR encoding of the instance to an encoder.

Required Methods

fn scheme() -> &'static str

The scheme / name for this SJS implementation.

This is advertised as the first component in the encoded SJS.

Provided Methods

fn to_bytes(&self) -> Result<Vec<u8>, RemoteSignError>

Obtain the raw bytes constituting the session join string.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors