Struct apple_codesign::app_store_connect::UnifiedApiKey
source · [−]pub struct UnifiedApiKey { /* private fields */ }Expand description
Represents all metadata for an App Store Connect API Key.
This is a convenience type to aid in the generic representation of all the components of an App Store Connect API Key. The type supports serialization so we save as a single file or payload to enhance usability (so people don’t need to provide all 3 pieces of the API Key for all operations).
Implementations
sourceimpl UnifiedApiKey
impl UnifiedApiKey
sourcepub fn from_ecdsa_pem_path(
issuer_id: impl ToString,
key_id: impl ToString,
path: impl AsRef<Path>
) -> Result<Self, AppleCodesignError>
pub fn from_ecdsa_pem_path(
issuer_id: impl ToString,
key_id: impl ToString,
path: impl AsRef<Path>
) -> Result<Self, AppleCodesignError>
Construct an instance from constitute parts and a PEM encoded ECDSA private key.
This is what you want to use if importing a private key from the file downloaded from the App Store Connect web interface.
sourcepub fn from_json(data: impl AsRef<[u8]>) -> Result<Self, AppleCodesignError>
pub fn from_json(data: impl AsRef<[u8]>) -> Result<Self, AppleCodesignError>
Construct an instance from serialized JSON.
sourcepub fn from_json_path(
path: impl AsRef<Path>
) -> Result<Self, AppleCodesignError>
pub fn from_json_path(
path: impl AsRef<Path>
) -> Result<Self, AppleCodesignError>
Construct an instance from a JSON file.
sourcepub fn to_json_string(&self) -> Result<String, AppleCodesignError>
pub fn to_json_string(&self) -> Result<String, AppleCodesignError>
Serialize this instance to a JSON object.
sourcepub fn write_json_file(
&self,
path: impl AsRef<Path>
) -> Result<(), AppleCodesignError>
pub fn write_json_file(
&self,
path: impl AsRef<Path>
) -> Result<(), AppleCodesignError>
Write this instance to a JSON file.
Since the file contains sensitive data, it will have limited read permissions on platforms where this is implemented. Parent directories will be created if missing using default permissions for created directories.
Permissions on the resulting file may not be as restrictive as desired. It is up to callers to additionally harden as desired.