Struct apple_codesign::notarization::Notarizer
source · pub struct Notarizer { /* private fields */ }Expand description
An entity for performing notarizations.
Notarization works by uploading content to Apple, waiting for Apple to inspect and react to that upload, then downloading a notarization “ticket” from Apple and incorporating it into the entity being signed.
Implementations
sourceimpl Notarizer
impl Notarizer
sourcepub fn from_api_key_id(
issuer_id: impl ToString,
key_id: impl ToString
) -> Result<Self, AppleCodesignError>
pub fn from_api_key_id(
issuer_id: impl ToString,
key_id: impl ToString
) -> Result<Self, AppleCodesignError>
Construct an instance from an API issuer ID and API key.
Examples found in repository?
1899 1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911
fn notarizer_from_args(args: &ArgMatches) -> Result<Notarizer, AppleCodesignError> {
let api_key_path = args.get_one::<PathBuf>("api_key_path");
let api_issuer = args.get_one::<String>("api_issuer");
let api_key = args.get_one::<String>("api_key");
if let Some(api_key_path) = api_key_path {
Notarizer::from_api_key(api_key_path)
} else if let (Some(issuer), Some(key)) = (api_issuer, api_key) {
Notarizer::from_api_key_id(issuer, key)
} else {
Err(AppleCodesignError::NotarizeNoAuthCredentials)
}
}sourcepub fn from_api_key(path: &Path) -> Result<Self, AppleCodesignError>
pub fn from_api_key(path: &Path) -> Result<Self, AppleCodesignError>
Construct an instance from a file containing a JSON encoded API key.
Examples found in repository?
1899 1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911
fn notarizer_from_args(args: &ArgMatches) -> Result<Notarizer, AppleCodesignError> {
let api_key_path = args.get_one::<PathBuf>("api_key_path");
let api_issuer = args.get_one::<String>("api_issuer");
let api_key = args.get_one::<String>("api_key");
if let Some(api_key_path) = api_key_path {
Notarizer::from_api_key(api_key_path)
} else if let (Some(issuer), Some(key)) = (api_issuer, api_key) {
Notarizer::from_api_key_id(issuer, key)
} else {
Err(AppleCodesignError::NotarizeNoAuthCredentials)
}
}sourcepub fn notarize_path(
&self,
path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
pub fn notarize_path(
&self,
path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
Attempt to notarize an asset defined by a filesystem path.
The type of path is sniffed out and the appropriate notarization routine is called.
Examples found in repository?
1938 1939 1940 1941 1942 1943 1944 1945 1946 1947 1948 1949 1950 1951 1952 1953 1954 1955 1956 1957 1958 1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 1970
fn command_notary_submit(args: &ArgMatches) -> Result<(), AppleCodesignError> {
let path = PathBuf::from(
args.get_one::<String>("path")
.expect("clap should have validated arguments"),
);
let staple = args.get_flag("staple");
let wait = args.get_flag("wait") || staple;
let wait_limit = if wait {
Some(notarizer_wait_duration(args)?)
} else {
None
};
let notarizer = notarizer_from_args(args)?;
let upload = notarizer.notarize_path(&path, wait_limit)?;
if staple {
match upload {
crate::notarization::NotarizationUpload::UploadId(_) => {
panic!(
"NotarizationUpload::UploadId should not be returned if we waited successfully"
);
}
crate::notarization::NotarizationUpload::NotaryResponse(_) => {
let stapler = crate::stapling::Stapler::new()?;
stapler.staple_path(&path)?;
}
}
}
Ok(())
}sourcepub fn notarize_bundle(
&self,
bundle: &DirectoryBundle,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
pub fn notarize_bundle(
&self,
bundle: &DirectoryBundle,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
Attempt to notarize an on-disk bundle.
If wait_limit is provided, we will wait for the upload to finish processing.
Otherwise, this returns as soon as the upload is performed.
Examples found in repository?
175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193
pub fn notarize_path(
&self,
path: &Path,
wait_limit: Option<Duration>,
) -> Result<NotarizationUpload, AppleCodesignError> {
match PathType::from_path(path)? {
PathType::Bundle => {
let bundle = DirectoryBundle::new_from_path(path)
.map_err(AppleCodesignError::DirectoryBundle)?;
self.notarize_bundle(&bundle, wait_limit)
}
PathType::Xar => self.notarize_flat_package(path, wait_limit),
PathType::Zip => self.notarize_flat_package(path, wait_limit),
PathType::Dmg => self.notarize_dmg(path, wait_limit),
PathType::MachO | PathType::Other => Err(AppleCodesignError::NotarizeUnsupportedPath(
path.to_path_buf(),
)),
}
}sourcepub fn notarize_dmg(
&self,
dmg_path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
pub fn notarize_dmg(
&self,
dmg_path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
Attempt to notarize a DMG file.
Examples found in repository?
175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193
pub fn notarize_path(
&self,
path: &Path,
wait_limit: Option<Duration>,
) -> Result<NotarizationUpload, AppleCodesignError> {
match PathType::from_path(path)? {
PathType::Bundle => {
let bundle = DirectoryBundle::new_from_path(path)
.map_err(AppleCodesignError::DirectoryBundle)?;
self.notarize_bundle(&bundle, wait_limit)
}
PathType::Xar => self.notarize_flat_package(path, wait_limit),
PathType::Zip => self.notarize_flat_package(path, wait_limit),
PathType::Dmg => self.notarize_dmg(path, wait_limit),
PathType::MachO | PathType::Other => Err(AppleCodesignError::NotarizeUnsupportedPath(
path.to_path_buf(),
)),
}
}sourcepub fn notarize_flat_package(
&self,
pkg_path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
pub fn notarize_flat_package(
&self,
pkg_path: &Path,
wait_limit: Option<Duration>
) -> Result<NotarizationUpload, AppleCodesignError>
Attempt to notarize a flat package (.pkg) installer or a .zip file.
Examples found in repository?
175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193
pub fn notarize_path(
&self,
path: &Path,
wait_limit: Option<Duration>,
) -> Result<NotarizationUpload, AppleCodesignError> {
match PathType::from_path(path)? {
PathType::Bundle => {
let bundle = DirectoryBundle::new_from_path(path)
.map_err(AppleCodesignError::DirectoryBundle)?;
self.notarize_bundle(&bundle, wait_limit)
}
PathType::Xar => self.notarize_flat_package(path, wait_limit),
PathType::Zip => self.notarize_flat_package(path, wait_limit),
PathType::Dmg => self.notarize_dmg(path, wait_limit),
PathType::MachO | PathType::Other => Err(AppleCodesignError::NotarizeUnsupportedPath(
path.to_path_buf(),
)),
}
}sourceimpl Notarizer
impl Notarizer
sourcepub fn get_submission(
&self,
submission_id: &str
) -> Result<SubmissionResponse, AppleCodesignError>
pub fn get_submission(
&self,
submission_id: &str
) -> Result<SubmissionResponse, AppleCodesignError>
Examples found in repository?
367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404
pub fn wait_on_notarization(
&self,
submission_id: &str,
wait_limit: Duration,
) -> Result<notary_api::SubmissionResponse, AppleCodesignError> {
warn!(
"waiting up to {}s for package upload {} to finish processing",
wait_limit.as_secs(),
submission_id
);
let start_time = std::time::Instant::now();
loop {
let status = self.get_submission(submission_id)?;
let elapsed = start_time.elapsed();
info!(
"poll state after {}s: {:?}",
elapsed.as_secs(),
status.data.attributes.status
);
if status.data.attributes.status != notary_api::SubmissionResponseStatus::InProgress {
warn!("Notary API Server has finished processing the uploaded asset");
return Ok(status);
}
if elapsed >= wait_limit {
warn!("reached wait limit after {}s", elapsed.as_secs());
return Err(AppleCodesignError::NotarizeWaitLimitReached);
}
std::thread::sleep(self.wait_poll_interval);
}
}sourcepub fn wait_on_notarization(
&self,
submission_id: &str,
wait_limit: Duration
) -> Result<SubmissionResponse, AppleCodesignError>
pub fn wait_on_notarization(
&self,
submission_id: &str,
wait_limit: Duration
) -> Result<SubmissionResponse, AppleCodesignError>
Examples found in repository?
419 420 421 422 423 424 425 426 427 428 429 430 431 432 433
pub fn wait_on_notarization_and_fetch_log(
&self,
submission_id: &str,
wait_limit: Duration,
) -> Result<notary_api::SubmissionResponse, AppleCodesignError> {
let status = self.wait_on_notarization(submission_id, wait_limit)?;
let log = self.fetch_notarization_log(submission_id)?;
for line in serde_json::to_string_pretty(&log)?.lines() {
warn!("notary log> {}", line);
}
Ok(status)
}sourcepub fn fetch_notarization_log(
&self,
submission_id: &str
) -> Result<Value, AppleCodesignError>
pub fn fetch_notarization_log(
&self,
submission_id: &str
) -> Result<Value, AppleCodesignError>
Obtain the processing log from an upload.
Examples found in repository?
1923 1924 1925 1926 1927 1928 1929 1930 1931 1932 1933 1934 1935 1936
fn command_notary_log(args: &ArgMatches) -> Result<(), AppleCodesignError> {
let notarizer = notarizer_from_args(args)?;
let submission_id = args
.get_one::<String>("submission_id")
.expect("submission_id is required");
let log = notarizer.fetch_notarization_log(submission_id)?;
for line in serde_json::to_string_pretty(&log)?.lines() {
println!("{line}");
}
Ok(())
}More examples
419 420 421 422 423 424 425 426 427 428 429 430 431 432 433
pub fn wait_on_notarization_and_fetch_log(
&self,
submission_id: &str,
wait_limit: Duration,
) -> Result<notary_api::SubmissionResponse, AppleCodesignError> {
let status = self.wait_on_notarization(submission_id, wait_limit)?;
let log = self.fetch_notarization_log(submission_id)?;
for line in serde_json::to_string_pretty(&log)?.lines() {
warn!("notary log> {}", line);
}
Ok(status)
}sourcepub fn wait_on_notarization_and_fetch_log(
&self,
submission_id: &str,
wait_limit: Duration
) -> Result<SubmissionResponse, AppleCodesignError>
pub fn wait_on_notarization_and_fetch_log(
&self,
submission_id: &str,
wait_limit: Duration
) -> Result<SubmissionResponse, AppleCodesignError>
Waits on an app store package upload and fetches and logs the upload log.
This is just a convenience around [Self::wait_on_app_store_package_upload()] and [Self::fetch_upload_log()].
Examples found in repository?
More examples
340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358
fn upload_s3_and_maybe_wait(
&self,
submission: notary_api::NewSubmissionResponse,
upload_data: UploadKind,
wait_limit: Option<Duration>,
) -> Result<NotarizationUpload, AppleCodesignError> {
self.upload_s3_package(&submission, upload_data)?;
let status = if let Some(wait_limit) = wait_limit {
self.wait_on_notarization_and_fetch_log(&submission.data.id, wait_limit)?
} else {
return Ok(NotarizationUpload::UploadId(submission.data.id));
};
// Make sure notarization was successful.
let status = status.into_result()?;
Ok(NotarizationUpload::NotaryResponse(status))
}Trait Implementations