Remote signing support.
Modules
- session_
negotiation - Session establishment and crypto code for remote signing protocol.
Structs
- Initiator
Client - A client fulfilling the role of the initiator.
- Paired
Client - A remote signing client that has joined a session and is ready to exchange messages.
- Signing
Client - Unjoined
Signing Client - A client for the remote signing protocol that has not yet joined a session.
Enums
- Remote
Sign Error - An error specific to remote signing.
Constants
- DEFAULT_
SERVER_ URL - URL of default server to use.
Type Aliases
- Session
Info Callback - A function that receives session information.